import 'package:flutter/material.dart';

import 'jumping_dots.dart';

class JumpingDotsProgressIndicator extends StatefulWidget {
  final int numberOfDots;
  final Duration animatedDuration;
  final Color dotColor;
  final double dotSize;

  const JumpingDotsProgressIndicator({
    Key? key,
    this.numberOfDots = 3,
    this.animatedDuration = const Duration(milliseconds: 250),
    this.dotColor = Colors.black,
    this.dotSize = 10,
  }) : super(key: key);

  @override
  _JumpingDotsProgressIndicatorState createState() =>
      _JumpingDotsProgressIndicatorState();
}

class _JumpingDotsProgressIndicatorState
    extends State<JumpingDotsProgressIndicator> with TickerProviderStateMixin {
  final double beginTweenValue = 0.0;
  final double endTweenValue = 20.0;
  List<AnimationController> controllers = <AnimationController>[];
  List<Animation<double>> animations = <Animation<double>>[];
  late List<Widget> widgets = <Widget>[];

  @override
  initState() {
    super.initState();
    for (int i = 0; i < widget.numberOfDots; i++) {
// adding controllers
      controllers.add(
          AnimationController(duration: widget.animatedDuration, vsync: this));
// adding animation values
      animations.add(Tween(begin: beginTweenValue, end: endTweenValue)
          .animate(controllers[i])
        ..addStatusListener((AnimationStatus status) {
          if (status == AnimationStatus.completed) controllers[i].reverse();
          if (i == widget.numberOfDots - 1 &&
              status == AnimationStatus.dismissed) {
            controllers[0].forward();
          }
          if (animations[i].value > endTweenValue / 2 &&
              i < widget.numberOfDots - 1) {
            controllers[i + 1].forward();
          }
        }));
// adding list of widgets
      widgets.add(Padding(
        padding: const EdgeInsets.only(right: 1.0),
        child: JumpingDot(
          animation: animations[i],
          dotSize: widget.dotSize,
          dotColor: widget.dotColor,
        ),
      ));
    }
// animating first dot in the list
    controllers[0].forward();
  }

  @override
  dispose() {
    for (int i = 0; i < widget.numberOfDots; i++) {
      controllers[i].dispose();
    }
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      height: 60.0,
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: widgets,
      ),
    );
  }
}
